/* This file includes miscellaneous functions that shouldn't be in gameserv.cpp */
#include "extern.h"
#include "flags.h"
#include "aClient.h"
#include "pouch.h"
#include "item.h"
#include "options.h"

#include <list>
#include <algorithm>
using namespace std;

void see_master(char *u)
{
  aClient *user;

  if (!(user = find(u)))
    {
	  notice(s_GameServ, u, "Fatal error. Contact a(n) <S admin. buf: %s", strtok(NULL, ""));
	  return;	
    }
  
  if (!is_fighting(user) && is_playing(user))
    {
	  Player *p = user->stats;
  	  p->setMyMaster(&levels[p->getLevel() - 1].master);
	  p->setMonster(&levels[p->getLevel() - 1].master);
	  display_monster(u);  // Since master is the same structure, use this function
    }
}

void rolloverall()
{
  list<Player*>::iterator iter;
  for (unsigned long x = 0; x < U_TABLE_SIZE; x++)
    {
      for (iter = players[x].begin(); iter != players[x].end(); iter++)
		{
		  rollover((*iter));
		}
    }
}
void refreshall()
{
  list<Player*>::iterator iter;
  for (unsigned long x = 0; x < U_TABLE_SIZE; x++)
    {
      for (iter = players[x].begin(); iter != players[x].end(); iter++)
		{
		  refresh((*iter));
		}
    }
}

void rollover(Player *p)
{
  if (!p)
    return;
  
  p->addForestFights(numrolloverfights);
  
  if (p->getForestFights() > maxforestfights)
    p->setForestFights(maxforestfights);
}

void refresh(Player *p)
{
  if (!p)
	return;

  if (p->getHP() < p->getMaxHP())
	p->healall();
  if (p->getForestFights() < forestfights)
	{
	  p->setForestFights(forestfights);
	}
  
  p->setPlayerFights(3);
  setAlive(p);
  clearMaster(p);
}




void resetall()
{
  list<Player*>::iterator iter;
  for (unsigned long x = 0; x < U_TABLE_SIZE; x++)
	{
	  for (iter = players[x].begin(); iter != players[x].end(); iter++)
		{
		  reset((*iter));
		}
	}
}

void reset(Player *p)
{
  item *tempItem;

  if (!p)
	return;
  
  p->reset();
  // Add the stick and clothes
  tempItem = findItemByID(3001);
  p->inventory->addItem((*Items.begin()))->use(p); // Add the stick
  p->inventory->addItem(tempItem)->use(p); // Add Clothes	
}

void updateTS(Player *p)
{
  if (!p)
	return;
  
#ifdef DEBUGMODE
  log("Old timestamp for %s: %ld", p->getName().c_str(), p->lastcommand);
#endif
  p->lastcommand = time(NULL);
#ifdef DEBUGMODE
  log("New timestamp for %s: %ld", p->getName().c_str(), p->lastcommand);
#endif
  PF_cleartimedout(p);
  
}



void timeOutEvent(Player *p)
{
  if (!p || !is_playing(p->getClient())) // then they're not playing
	return;
  
  char *nick = p->getClient()->getNick();

  if (player_fight(p->getClient()) && isYourTurn(p))
    {
	  // Check to see if they were the idler or if it was the other 
	  // person
	  if (!PF_timedout(p->getBattle()->stats))
		{
		  // This person's last command was given earlier,
		  // so this person is the idler
		  notice(s_GameServ, nick, "You timed out "\
				 "during a fight. You lose your turn!");
		  notice(s_GameServ, p->getBattle()->getNick(),
				 "%s hesitated for too long. Your move.", p->getName().c_str());
		  clearYourTurn(p);
		  setYourTurn(p->getBattle()->stats);
		  
		  // Update the TS for both players to give them another
		  // Chance to wake up, but if the other player doesn't
		  // Attack now, they both get logged out.
		  updateTS(p);
		  PF_settimedout(p);
		  display_players(p->getBattle());
		  return;
		}
	  else
		{
		  notice(s_GameServ, p->getBattle()->getNick(),
				 "You and %s timed out at the same time."\
				 " Don't fight if you're just going to "\
				 "sit there!", p->getName().c_str());
		  notice(s_GameServ, p->getClient()->getNick(),
				 "You and %s timed out at the same time."\
				 " Don't fight if you're just going to "\
				 "sit there!", p->getBattle()->stats->getName().c_str());
		  logout(p->getBattle());
		  logout(p->getClient());
		  clearYourTurn(p);
		  clearYourTurn(p->getBattle()->stats);
		  return;
		}
    }
  else if (!player_fight(p->getClient()))
    {
	  if (isAlive(p) && p->getGold() > 0)
		{
		  // Place fun stuff here :)
		  int randnum = 1 + rand() % 100; // 1-100
#define GSN(s) notice(s_GameServ, nick, s)
#define GSN2(s, f) notice(s_GameServ, nick, s, f)
		  
		  if (randnum < 50)
			{
			  // 35-100% of your gold goes pffft - kain
			  int stolen = (35 + (rand() % 66)) * (p->getGold() / 100);
			  
			  GSN("You stop for a moment to rest on the "\
				  "street corner. All of a sudden, you "\
				  "are ambushed from all sides by a hoarde "\
				  "of knife wielding thugs.");
			  GSN2("The thugs beat you into utter submission "\
				   "and steal %ld gold from you!", stolen);
			  p->subtractGold(stolen);
			}
		  else if (randnum >= 50 && randnum < 75)
			{
			  // 25-65% of your gold goes pffft - kain
			  int stolen = (25 + (rand() % 41)) * (p->getGold() / 100);
			  GSN("While dilly dallying around, you lose "\
				  "your sense of time. Little did you know, "\
				  "but thieves lifted your gold while you "\
				  "weren't watching.");
			  GSN2("Better luck next time... you lose %d gold", stolen);
			  p->subtractGold(stolen);
			}
		  else if (randnum >= 75)
			{
			  // 25-75% of your gold goes pffft - kain
			  int stolen = (25 + (rand() % 51)) * (p->getGold() / 100);
			  GSN("Good grief! A gaggle of gooey green ghostlike "\
				  "goblins grabbed your gold!");
			  GSN2("They stole %d gold from you!", stolen);
			  p->subtractGold(stolen);
			}
		}
	  
	  // Always log out the user
	  logout(p->getClient());
    }
}
void logout(aClient *user)
{
  if (user != NULL)
    {
	  if (user->stats != NULL)
		{
		
		  list<Player*>::iterator iter;
		  unsigned long hv = iHASH((unsigned char *) user->stats->getName().c_str());
		  iter = find(players[hv].begin(), players[hv].end(), user->stats);
		  
		  if (iter == players[hv].end())
			{
			  notice(s_GameServ, user->getNick(), "Fatal error. Contact "\
					 "<S Admin. Cannot find you in the players list. This should NEVER happen");
			  log("Error on logout(). Can't find %s in the players list",
#ifdef P10 
				  user->getRealNick()
#else 
				  user->getNick()
#endif
				  );
			  user->stats = NULL;
			  
			  return;
			}
		  user->stats->delMonster();
		  user->stats->delMaster();
		  user->stats->delBattle();
		  clearDragonFight(user->stats);
		  clearYourTurn(user->stats);
		  clearPlaying(user);
		  
		  user->stats->setClient(NULL);
		  
		  
		  if (player_fight(user))
			{
			  clearYourTurn(user->stats->getBattle()->stats);
			  user->stats->getBattle()->stats->delBattle();
			}
		  
#ifdef DEBUGMODE
		  log("Logged out player %s",
#ifdef P10 
			  user->getRealNick()
#else 
			  user->getNick()
#endif 
			  );
#endif
		}
	}
  if (user)
	user->stats = NULL;
}

void end_turn(aClient *user)
{
  char *nick, *u = user->getNick();
  Monster *fight;
  aClient *battle;
  int mhit;
  
  nick = new char[strlen(user->getNick()) + 1];
  
  if (!user || !is_playing(user) || !is_fighting(user))
	goto endturn;
  
  if (!player_fight(user) && !master_fight(user))
	fight = user->stats->getMonster();
  else
	fight = user->stats->getMaster();
  battle = user->stats->getBattle();
  
  if (!player_fight(user))
    {
	  // Opponent's Hit
	  mhit = (fight->strength / 2) + (rand() % (fight->strength / 2)) - (user->stats->getDefense());
    }
  else
    {
	  // Opponent's Hit
	  mhit = (battle->stats->getStrength() / 2) + (rand() % (battle->stats->getStrength() / 2)) - user->stats->getDefense();
    }
  if (!player_fight(user))
    {
	  
	  if (mhit > 0)
        {
		  notice(s_GameServ, u, "%s attacks with their %s for %d damage!",
				 fight->name.c_str(), fight->weapon.c_str(), mhit);
        }
	  else if (mhit <= 0)
		notice(s_GameServ, u, "%s completely misses you!", fight->name.c_str());
	  
	  if (mhit >= user->stats->getHP())
        {
		  if (!master_fight(user))
            {
			  notice(s_GameServ, u, "You have been killed by %s!", fight->name.c_str());
			  notice(s_GameServ, u, "You lose all gold on hand and lose 10 percent "\
					 "of your experience!");
			  user->stats->setGold(0);
			  user->stats->subtractExp((long int)(user->stats->getExp() * .10));
			  user->stats->setHP(0);
			  user->stats->delMonster();
			  clearAlive(user->stats);
			  goto endturn;
            }
		  else
            {
			  notice(s_GameServ, u, "%s has bested you! You will have to wait "\
					 "until tomorrow to try again", user->stats->getMaster()->name.c_str());
			  user->stats->delMonster();
			  user->stats->delMaster();
			  goto endturn;
            }
        }
	  else
        {
		  if (mhit > 0)
			user->stats->subtractHP(mhit);
		  display_monster(u);
		  goto endturn;
        }
    }
  else
    {
	  clearYourTurn(user->stats);
	  setYourTurn(battle->stats);
	  display_players(battle);
    }
 endturn:
  delete []nick;
}
